# Many definitions are not included in http.client.__all__
from http.client import *
from http.client import (
    ACCEPTED as ACCEPTED,
    BAD_GATEWAY as BAD_GATEWAY,
    BAD_REQUEST as BAD_REQUEST,
    CONFLICT as CONFLICT,
    CONTINUE as CONTINUE,
    CREATED as CREATED,
    EXPECTATION_FAILED as EXPECTATION_FAILED,
    FAILED_DEPENDENCY as FAILED_DEPENDENCY,
    FORBIDDEN as FORBIDDEN,
    FOUND as FOUND,
    GATEWAY_TIMEOUT as GATEWAY_TIMEOUT,
    GONE as GONE,
    HTTP_PORT as HTTP_PORT,
    HTTP_VERSION_NOT_SUPPORTED as HTTP_VERSION_NOT_SUPPORTED,
    HTTPS_PORT as HTTPS_PORT,
    IM_USED as IM_USED,
    INSUFFICIENT_STORAGE as INSUFFICIENT_STORAGE,
    INTERNAL_SERVER_ERROR as INTERNAL_SERVER_ERROR,
    LENGTH_REQUIRED as LENGTH_REQUIRED,
    LOCKED as LOCKED,
    METHOD_NOT_ALLOWED as METHOD_NOT_ALLOWED,
    MOVED_PERMANENTLY as MOVED_PERMANENTLY,
    MULTI_STATUS as MULTI_STATUS,
    MULTIPLE_CHOICES as MULTIPLE_CHOICES,
    NETWORK_AUTHENTICATION_REQUIRED as NETWORK_AUTHENTICATION_REQUIRED,
    NO_CONTENT as NO_CONTENT,
    NON_AUTHORITATIVE_INFORMATION as NON_AUTHORITATIVE_INFORMATION,
    NOT_ACCEPTABLE as NOT_ACCEPTABLE,
    NOT_EXTENDED as NOT_EXTENDED,
    NOT_FOUND as NOT_FOUND,
    NOT_IMPLEMENTED as NOT_IMPLEMENTED,
    NOT_MODIFIED as NOT_MODIFIED,
    OK as OK,
    PARTIAL_CONTENT as PARTIAL_CONTENT,
    PAYMENT_REQUIRED as PAYMENT_REQUIRED,
    PRECONDITION_FAILED as PRECONDITION_FAILED,
    PRECONDITION_REQUIRED as PRECONDITION_REQUIRED,
    PROCESSING as PROCESSING,
    PROXY_AUTHENTICATION_REQUIRED as PROXY_AUTHENTICATION_REQUIRED,
    REQUEST_ENTITY_TOO_LARGE as REQUEST_ENTITY_TOO_LARGE,
    REQUEST_HEADER_FIELDS_TOO_LARGE as REQUEST_HEADER_FIELDS_TOO_LARGE,
    REQUEST_TIMEOUT as REQUEST_TIMEOUT,
    REQUEST_URI_TOO_LONG as REQUEST_URI_TOO_LONG,
    REQUESTED_RANGE_NOT_SATISFIABLE as REQUESTED_RANGE_NOT_SATISFIABLE,
    RESET_CONTENT as RESET_CONTENT,
    SEE_OTHER as SEE_OTHER,
    SERVICE_UNAVAILABLE as SERVICE_UNAVAILABLE,
    SWITCHING_PROTOCOLS as SWITCHING_PROTOCOLS,
    TEMPORARY_REDIRECT as TEMPORARY_REDIRECT,
    TOO_MANY_REQUESTS as TOO_MANY_REQUESTS,
    UNAUTHORIZED as UNAUTHORIZED,
    UNPROCESSABLE_ENTITY as UNPROCESSABLE_ENTITY,
    UNSUPPORTED_MEDIA_TYPE as UNSUPPORTED_MEDIA_TYPE,
    UPGRADE_REQUIRED as UPGRADE_REQUIRED,
    USE_PROXY as USE_PROXY,
    HTTPMessage as HTTPMessage,
    parse_headers as parse_headers,
)



import builtins
import operator
import types
import unittest
from _typeshed import IdentityFunction, _KT_contra, _VT_co
from builtins import next as next
from collections.abc import Callable, ItemsView, Iterable, Iterator as _Iterator, KeysView, Mapping, ValuesView
from functools import wraps as wraps
from importlib.util import spec_from_loader as spec_from_loader
from io import BytesIO as BytesIO, StringIO as StringIO
from re import Pattern
from typing import Any, AnyStr, NoReturn, Protocol, TypeVar, overload
from typing_extensions import Literal

from six import moves as moves

# TODO: We should switch to the _typeshed version of SupportsGetItem
# once mypy updates its vendored copy of typeshed and makes a new release
class _SupportsGetItem(Protocol[_KT_contra, _VT_co]):
    def __contains__(self, __x: Any) -> bool: ...
    def __getitem__(self, __key: _KT_contra) -> _VT_co: ...

_T = TypeVar("_T")
_K = TypeVar("_K")
_V = TypeVar("_V")

__author__: str
__version__: str

PY2: Literal[False]
PY3: Literal[True]
PY34: Literal[True]

string_types: tuple[type[str]]
integer_types: tuple[type[int]]
class_types: tuple[type[type]]
text_type = str
binary_type = bytes

MAXSIZE: int

callable = builtins.callable

def get_unbound_function(unbound: types.FunctionType) -> types.FunctionType: ...

create_bound_method = types.MethodType

def create_unbound_method(func: types.FunctionType, cls: type) -> types.FunctionType: ...

Iterator = object

def get_method_function(meth: types.MethodType) -> types.FunctionType: ...
def get_method_self(meth: types.MethodType) -> object | None: ...
def get_function_closure(fun: types.FunctionType) -> tuple[types._Cell, ...] | None: ...
def get_function_code(fun: types.FunctionType) -> types.CodeType: ...
def get_function_defaults(fun: types.FunctionType) -> tuple[Any, ...] | None: ...
def get_function_globals(fun: types.FunctionType) -> dict[str, Any]: ...
def iterkeys(d: Mapping[_K, Any]) -> _Iterator[_K]: ...
def itervalues(d: Mapping[Any, _V]) -> _Iterator[_V]: ...
def iteritems(d: Mapping[_K, _V]) -> _Iterator[tuple[_K, _V]]: ...
def viewkeys(d: Mapping[_K, Any]) -> KeysView[_K]: ...
def viewvalues(d: Mapping[Any, _V]) -> ValuesView[_V]: ...
def viewitems(d: Mapping[_K, _V]) -> ItemsView[_K, _V]: ...
def b(s: str) -> bytes: ...
def u(s: str) -> str: ...

unichr = chr

def int2byte(i: int) -> bytes: ...

# Should be `byte2int: operator.itemgetter[int]`. But a bug in mypy prevents using TypeVar in itemgetter.__call__
def byte2int(obj: _SupportsGetItem[int, _T]) -> _T: ...

indexbytes = operator.getitem
iterbytes = iter

def assertCountEqual(self: unittest.TestCase, first: Iterable[_T], second: Iterable[_T], msg: str | None = ...) -> None: ...
@overload
def assertRaisesRegex(self: unittest.TestCase, msg: str | None = ...) -> Any: ...
@overload
def assertRaisesRegex(self: unittest.TestCase, callable_obj: Callable[..., object], *args: Any, **kwargs: Any) -> Any: ...
def assertRegex(self: unittest.TestCase, text: AnyStr, expected_regex: AnyStr | Pattern[AnyStr], msg: Any = ...) -> None: ...
def assertNotRegex(self: unittest.TestCase, text: AnyStr, expected_regex: AnyStr | Pattern[AnyStr], msg: Any = ...) -> None: ...

exec_ = exec

def reraise(tp: type[BaseException] | None, value: BaseException | None, tb: types.TracebackType | None = ...) -> NoReturn: ...
def raise_from(value: BaseException | type[BaseException], from_value: BaseException | None) -> NoReturn: ...

print_ = print

def with_metaclass(meta: type, *bases: type) -> type: ...
def add_metaclass(metaclass: type) -> IdentityFunction: ...
def ensure_binary(s: bytes | str, encoding: str = ..., errors: str = ...) -> bytes: ...
def ensure_str(s: bytes | str, encoding: str = ..., errors: str = ...) -> str: ...
def ensure_text(s: bytes | str, encoding: str = ..., errors: str = ...) -> str: ...
def python_2_unicode_compatible(klass: _T) -> _T: ...

class _LazyDescr:
    name: str
    def __init__(self, name: str) -> None: ...
    def __get__(self, obj: object | None, tp: object) -> Any: ...

class MovedModule(_LazyDescr):
    mod: str
    def __init__(self, name: str, old: str, new: str | None = ...) -> None: ...
    def __getattr__(self, attr: str) -> Any: ...

class MovedAttribute(_LazyDescr):
    mod: str
    attr: str
    def __init__(self, name: str, old_mod: str, new_mod: str, old_attr: str | None = ..., new_attr: str | None = ...) -> None: ...

def add_move(move: MovedModule | MovedAttribute) -> None: ...
def remove_move(name: str) -> None: ...
